import pytz
from flask import Flask, render_template, jsonify, request

from tokenizer import resolve_query, QUERY_TYPE_DT, QUERY_TYPE_DT_TR, QUERY_TYPE_TZ, QUERY_TYPE_TD

app = Flask(__name__)

IN_KW = " in "
TO_KW = " to "
NO_TZ_FORMAT = '%Y-%m-%dT%H:%M:%S'
ISO_FORMAT = '%Y-%m-%dT%H:%M:%S%z'
DATE_FORMAT = '%Y-%m-%d'
TIME_FORMAT = '%H:%M:%S'
TZ_OFFSET_FORMAT = '%z'


@app.route("/")
def home():
    ctx = dict(all_tz=pytz.all_timezones)
    q = request.args.get('q')
    if q:
        ctx["q_resp"] = jsonify(resolve_query(q))
    return render_template("home.html", all_tz=pytz.all_timezones)


def render_solution(solution):
    query_type = solution["query_type"]
    if query_type == QUERY_TYPE_DT:
        return render_template("dt.html", **solution)
    elif query_type == QUERY_TYPE_DT_TR:
        return render_template("dt_tr.html", **solution)
    elif query_type == QUERY_TYPE_TZ:
        return render_template("tz.html", **solution)
    elif query_type == QUERY_TYPE_TD:
        return render_template("td.html", **solution)


def render_solutions(results):
    rv = []
    for solution in results['solutions']:
        rv.append(render_solution(solution))
    return rv


@app.route("/api/datetime")
def api_datetime():
    q = request.args.get('q')
    results = resolve_query(q)
    rendered_solutions = render_solutions(results)
    results['rendered_solutions'] = rendered_solutions
    return jsonify(results)


if __name__ == "__main__":
    app.run(debug=True)
